@@ -51,9 +51,7 @@ DecorationButton::Private::~Private() = default;
 
 void DecorationButton::Private::init()
 {
-    auto clientPtr = decoration->client().toStrongRef();
-    Q_ASSERT(clientPtr);
-    auto c = clientPtr.data();
+    auto c = decoration->client();
     auto settings = decoration->settings();
     switch (type) {
     case DecorationButtonType::Menu:
@@ -68,7 +66,7 @@ void DecorationButton::Private::init()
             Qt::QueuedConnection);
         QObject::connect(q, &DecorationButton::doubleClicked, decoration.data(), &Decoration::requestClose, Qt::QueuedConnection);
         QObject::connect(
-            settings.data(),
+            settings.get(),
             &DecorationSettings::closeOnDoubleClickOnMenuChanged,
             q,
             [this](bool enabled) {
@@ -100,7 +98,7 @@ void DecorationButton::Private::init()
         setCheckable(true);
         setChecked(c->isOnAllDesktops());
         QObject::connect(q, &DecorationButton::clicked, decoration.data(), &Decoration::requestToggleOnAllDesktops, Qt::QueuedConnection);
-        QObject::connect(settings.data(), &DecorationSettings::onAllDesktopsAvailableChanged, q, &DecorationButton::setVisible);
+        QObject::connect(settings.get(), &DecorationSettings::onAllDesktopsAvailableChanged, q, &DecorationButton::setVisible);
         QObject::connect(c, &DecoratedClient::onAllDesktopsChanged, q, &DecorationButton::setChecked);
         break;
     case DecorationButtonType::Minimize:
@@ -239,15 +237,15 @@ void DecorationButton::Private::startDoubleClickTimer()
     if (!doubleClickEnabled) {
         return;
     }
-    if (m_doubleClickTimer.isNull()) {
-        m_doubleClickTimer.reset(new QElapsedTimer());
+    if (!m_doubleClickTimer) {
+        m_doubleClickTimer = std::make_unique<QElapsedTimer>();
     }
     m_doubleClickTimer->start();
 }
 
 void DecorationButton::Private::invalidateDoubleClickTimer()
 {
-    if (m_doubleClickTimer.isNull()) {
+    if (!m_doubleClickTimer) {
         return;
     }
     m_doubleClickTimer->invalidate();
@@ -255,7 +253,7 @@ void DecorationButton::Private::invalidateDoubleClickTimer()
 
 bool DecorationButton::Private::wasDoubleClick() const
 {
-    if (m_doubleClickTimer.isNull() || !m_doubleClickTimer->isValid()) {
+    if (!m_doubleClickTimer || !m_doubleClickTimer->isValid()) {
         return false;
     }
     return !m_doubleClickTimer->hasExpired(QGuiApplication::styleHints()->mouseDoubleClickInterval());
@@ -277,10 +275,10 @@ void DecorationButton::Private::startPressAndHold()
     if (!pressAndHold) {
         return;
     }
-    if (m_pressAndHoldTimer.isNull()) {
+    if (!m_pressAndHoldTimer) {
         m_pressAndHoldTimer.reset(new QTimer());
         m_pressAndHoldTimer->setSingleShot(true);
-        QObject::connect(m_pressAndHoldTimer.data(), &QTimer::timeout, q, [this]() {
+        QObject::connect(m_pressAndHoldTimer.get(), &QTimer::timeout, q, [this]() {
             q->clicked(Qt::LeftButton);
         });
     }
@@ -289,7 +287,7 @@ void DecorationButton::Private::startPressAndHold()
 
 void DecorationButton::Private::stopPressAndHold()
 {
-    if (!m_pressAndHoldTimer.isNull()) {
+    if (m_pressAndHoldTimer) {
         m_pressAndHoldTimer->stop();
     }
 }
@@ -337,7 +335,7 @@ QString DecorationButton::Private::typeToString(DecorationButtonType type)
     }
 }
 
-DecorationButton::DecorationButton(DecorationButtonType type, const QPointer<Decoration> &decoration, QObject *parent)
+DecorationButton::DecorationButton(DecorationButtonType type, Decoration *decoration, QObject *parent)
     : QObject(parent)
     , d(new Private(type, decoration, this))
 {
@@ -415,7 +413,7 @@ DELEGATE(isVisible, visible, bool)
 
 #define DELEGATE2(name, type) DELEGATE(name, name, type)
 DELEGATE2(geometry, QRectF)
-DELEGATE2(decoration, QPointer<Decoration>)
+DELEGATE2(decoration, Decoration *)
 DELEGATE2(acceptedButtons, Qt::MouseButtons)
 DELEGATE2(type, DecorationButtonType)
 
@@ -565,3 +563,5 @@ void DecorationButton::wheelEvent(QWheelEvent *event)
 }
 
 }
+
+#include "moc_decorationbutton.cpp"
